/*
 * AVISA software - $Id: CalibratedBandReader.java,v 1.3 2006/03/13 14:02:04 marcoz Exp $
 *
 * Copyright (C) 2005 by EUMETSAT
 *
 * The Licensee acknowledges that the AVISA software is owned by the European
 * Organisation for the Exploitation of Meteorological Satellites
 * (EUMETSAT) and the Licensee shall not transfer, assign, sub-licence,
 * reproduce or copy the AVISA software to any third party or part with
 * possession of this software or any part thereof in any way whatsoever.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 * The AVISA software has been developed using the ESA BEAM software which is
 * distributed under the GNU General Public License (GPL).
 *
 */
package org.eumetsat.beam.dataio.metop;

import java.io.IOException;

import javax.imageio.stream.ImageInputStream;

import org.esa.beam.dataio.avhrr.AvhrrConstants;
import org.esa.beam.dataio.avhrr.AvhrrFile;
import org.esa.beam.dataio.avhrr.AvhrrReader;
import org.esa.beam.dataio.avhrr.calibration.RadianceCalibrator;
import org.esa.beam.framework.datamodel.ProductData;

import com.bc.progress.ProgressController;
import com.bc.progress.ProgressControllerPool;

/**
 * Reads radiances from the METOP prodcuts and transforms them into
 * a reflectance factor or a temperature.
 * 
* @author marcoz
 * @version $Revision: 1.3 $ $Date: 2006/03/13 14:02:04 $
 */
public class CalibratedBandReader extends PlainBandReader {

	short[] radianceScanLine;
	
	RadianceCalibrator calibrator;
	
	public CalibratedBandReader(int channel, MetopFile metopFile,
			ImageInputStream inputStream, RadianceCalibrator radianceCalibrator) {
		super(channel, metopFile, inputStream);
		calibrator = radianceCalibrator;
		radianceScanLine = new short[AvhrrConstants.SCENE_RASTER_WIDTH];
	}

	public String getBandName() {
		if (isVisibleBand()) {
			return AvhrrConstants.REFLECTANCE_BAND_NAME_PREFIX
					+ AvhrrConstants.CH_STRINGS[channel];
		} else {
			return AvhrrConstants.TEMPERATURE_BAND_NAME_PREFIX
					+ AvhrrConstants.CH_STRINGS[channel];
		}
	}

	public String getBandUnit() {
		if (isVisibleBand()) {
			return AvhrrConstants.REFLECTANCE_UNIT;
		} else {
			return AvhrrConstants.TEMPERATURE_UNIT;
		}
	}

	public String getBandDescription() {
		if (isVisibleBand()) {
			return AvhrrReader.format(
					AvhrrConstants.REFLECTANCE_FACTOR_DESCRIPTION,
					AvhrrConstants.CH_STRINGS[channel]);
		} else {
			return AvhrrReader.format(AvhrrConstants.TEMPERATURE_DESCRIPTION,
					AvhrrConstants.CH_STRINGS[channel]);
		}
	}

	public float getScalingFactor() {
		return 1;
	}

	public int getDataType() {
		return ProductData.TYPE_FLOAT32;
	}

	public void readBandRasterData(int sourceOffsetX, int sourceOffsetY,
			int sourceWidth, int sourceHeight, int sourceStepX,
			int sourceStepY, ProductData destBuffer) throws IOException {

		AvhrrFile.RawCoordinates rawCoord = metopFile.getRawCoordiantes(
				sourceOffsetX, sourceOffsetY, sourceWidth, sourceHeight);
		final float[] targetData = (float[]) destBuffer.getElems();
		final float scalingFactor = super.getScalingFactor();
		
		final ProgressController pc = ProgressControllerPool.getInstance()
				.getProgressController();
		pc.fireProcessStarted("Reading AVHRR band '" + getBandName() + "'...",
				rawCoord.minY, rawCoord.maxY); /* I18N */

		int targetIdx = rawCoord.targetStart;
		for (int sourceY = rawCoord.minY; sourceY <= rawCoord.maxY; sourceY += sourceStepY) {
			if (pc.isTerminationRequested()) {
				break;
			}

			if (hasData(sourceY)) {
				final int dataOffset = getDataOffset(sourceOffsetX, sourceY);
				synchronized (inputStream) {
					inputStream.seek(dataOffset);
					inputStream.readFully(radianceScanLine, 0, sourceWidth);
					
					for (int sourceX = 0; sourceX <= sourceWidth-1; sourceX++) {
						final float radiances = calibrator
								.calibrate(radianceScanLine[sourceX] * scalingFactor);
						targetData[targetIdx] = radiances;
						targetIdx += rawCoord.targetIncrement;
					}
				}
			} else {
				for (int sourceX = rawCoord.minX; sourceX <= rawCoord.maxX; sourceX += sourceStepX) {
					targetData[targetIdx] = AvhrrConstants.NO_DATA_VALUE;
					targetIdx += rawCoord.targetIncrement;
				}
			}
			pc.fireProcessInProgress(sourceY);
		}
		pc.fireProcessEnded();
	}
}
