/*
 * AVISA software - $Id: MetopFile.java,v 1.3 2006/03/13 14:02:05 marcoz Exp $
 *
 * Copyright (C) 2005 by EUMETSAT
 *
 * The Licensee acknowledges that the AVISA software is owned by the European
 * Organisation for the Exploitation of Meteorological Satellites
 * (EUMETSAT) and the Licensee shall not transfer, assign, sub-licence,
 * reproduce or copy the AVISA software to any third party or part with
 * possession of this software or any part thereof in any way whatsoever.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 * The AVISA software has been developed using the ESA BEAM software which is
 * distributed under the GNU General Public License (GPL).
 *
 */
package org.eumetsat.beam.dataio.metop;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.stream.FileImageInputStream;
import javax.imageio.stream.ImageInputStream;

import org.esa.beam.dataio.avhrr.AvhrrConstants;
import org.esa.beam.dataio.avhrr.AvhrrFile;
import org.esa.beam.dataio.avhrr.BandReader;
import org.esa.beam.dataio.avhrr.calibration.Radiance2ReflectanceFactorCalibrator;
import org.esa.beam.dataio.avhrr.calibration.Radiance2TemperatureCalibrator;
import org.esa.beam.dataio.avhrr.calibration.RadianceCalibrator;
import org.esa.beam.framework.datamodel.ProductData.UTC;

/**
 * Reads a product file containing a METOP-AVHRR/3 product.
 *
 * @author marcoz
 * @version $Revision: 1.3 $ $Date: 2006/03/13 14:02:05 $
 */
public class MetopFile extends AvhrrFile {

	private static final int MDR_SIZE = 25828;

	private ImageInputStream inputStream;

	private GenericRecordHeader mphrHeader;

	private AsciiRecord mainProductHeaderRecord;

	private AsciiRecord secondaryProductHeaderRecord;

	private GiadrRadiance giadrRadiance;

	private int firstMdrOffset;

	public MetopFile(ImageInputStream imageInputStream) {
		this.inputStream = imageInputStream;
	}

	public void readHeader() throws IOException {
		mphrHeader = new GenericRecordHeader();
		mphrHeader.readGenericRecordHeader(inputStream);

		if (mphrHeader.recordClass == GenericRecordHeader.RecordClass.MPHR
				&& mphrHeader.instrumentGroup == GenericRecordHeader.InstrumentGroup.GENERIC
				&& mphrHeader.recordSubclass == 0) {
			mainProductHeaderRecord = new MainProductHeaderRecord();
			mainProductHeaderRecord.readRecord(inputStream);
		} else {
			throw new IOException("Unsupported product: bad MPHR. RecordClass="
					+ mphrHeader.recordClass + " InstrumentGroup="
					+ mphrHeader.instrumentGroup + " RecordSubclass="
					+ mphrHeader.recordSubclass);
		}

		if (mainProductHeaderRecord.getIntValue("TOTAL_SPHR") != 1) {
			throw new IOException("Unsupported Product: more than one SPHR.");
		}
		GenericRecordHeader sphrHeader = new GenericRecordHeader();
		sphrHeader.readGenericRecordHeader(inputStream);

		if (sphrHeader.recordClass == GenericRecordHeader.RecordClass.SPHR
				&& sphrHeader.instrumentGroup == GenericRecordHeader.InstrumentGroup.AVHRR3
				&& sphrHeader.recordSubclass == 0) {
			secondaryProductHeaderRecord = new SecondaryProductHeaderRecord();
			secondaryProductHeaderRecord.readRecord(inputStream);
		} else {
			throw new IOException("Unsupported product: bad SPHR. RecordClass="
					+ sphrHeader.recordClass + " InstrumentGroup="
					+ sphrHeader.instrumentGroup + " RecordSubclass="
					+ sphrHeader.recordSubclass);
		}
		
		if(secondaryProductHeaderRecord.getIntValue("EARTH_VIEWS_PER_SCANLINE") != 2048) {
			throw new IOException("Unsupported product: bad SPHR. " +
					"EARTH_VIEWS_PER_SCANLINE is not 2048. Actual value: " +
					secondaryProductHeaderRecord.getIntValue("EARTH_VIEWS_PER_SCANLINE"));
		}
		if(secondaryProductHeaderRecord.getIntValue("NAV_SAMPLE_RATE") != 40) {
			throw new IOException("Unsupported product: bad SPHR. " +
					"NAV_SAMPLE_RATE is not 40. Actual value: " +
					secondaryProductHeaderRecord.getIntValue("NAV_SAMPLE_RATE"));
		}

		final int numIPR = mainProductHeaderRecord.getIntValue("TOTAL_IPR");
		InternalPointerRecord[] iprs = new InternalPointerRecord[numIPR];

		for (int i = 0; i < numIPR; i++) {
			iprs[i] = new InternalPointerRecord();
			iprs[i].readRecord(inputStream);
			iprs[i].printIPR();
		}

		for (int i = 0; i < numIPR; i++) {
			// read GIEDR
			if (iprs[i].targetRecordClass == GenericRecordHeader.RecordClass.GIADR
					&& iprs[i].targetRecordSubclass == 1) {
				giadrRadiance = new GiadrRadiance();
				giadrRadiance.readRecord(inputStream);
			}
			if (iprs[i].targetRecordClass == GenericRecordHeader.RecordClass.GIADR
					&& iprs[i].targetRecordSubclass == 2) {
				// GiedrAnalog
				// data not read
			}
			if (iprs[i].targetRecordClass == GenericRecordHeader.RecordClass.MDR) {
				firstMdrOffset = iprs[i].targetRecordOffset;
			}
		}
		productHeight = mainProductHeaderRecord.getIntValue("TOTAL_MDR");
		productWidth = AvhrrConstants.SCENE_RASTER_WIDTH;
		checkMdrs();
		analyzeFrameIndicator();
	}

	public String getProductName() {
		return mainProductHeaderRecord.getValue("PRODUCT_NAME");
	}

	public UTC getStartDate() {
		return mphrHeader.recordStartTime;
	}

	public UTC getEndDate() {
		return mphrHeader.recordEndTime;
	}

	public List getMetaData() {
		List metaDataList = new ArrayList();

		metaDataList.add(mainProductHeaderRecord.getMetaData());
		metaDataList.add(secondaryProductHeaderRecord.getMetaData());
		metaDataList.add(giadrRadiance.getMetaData());
		return metaDataList;
	}

	public BandReader createVisibleRadianceBandReader(int channel) {
		return new PlainBandReader(channel, this, inputStream);
	}

	public BandReader createIrRadianceBandReader(int channel) {
		return new PlainBandReader(channel, this, inputStream);
	}

	public BandReader createReflectanceFactorBandReader(int channel) {
		RadianceCalibrator radianceCalibrator = new Radiance2ReflectanceFactorCalibrator(
				giadrRadiance.getEquivalentWidth(channel), giadrRadiance
						.getSolarIrradiance(channel), 1);
		return new CalibratedBandReader(channel, this, inputStream,
				radianceCalibrator);
	}

	public BandReader createIrTemperatureBandReader(int channel) {
		RadianceCalibrator radianceCalibrator = new Radiance2TemperatureCalibrator(
				giadrRadiance.getConstant1(channel), giadrRadiance
						.getConstant2(channel), giadrRadiance
						.getCentralWavenumber(channel));
		return new CalibratedBandReader(channel, this, inputStream,
				radianceCalibrator);
	}

	public String[] getTiePointNames() {
		String[] names = { AvhrrConstants.SZA_DS_NAME,
				AvhrrConstants.VZA_DS_NAME, AvhrrConstants.SAA_DS_NAME,
				AvhrrConstants.VAA_DS_NAME, AvhrrConstants.LAT_DS_NAME,
				AvhrrConstants.LON_DS_NAME };
		return names;
	}

	public float[][] getTiePointData() throws IOException {
		final int gridHeight = getProductHeight()
				/ AvhrrConstants.TP_SUB_SAMPLING_Y + 1;
		final int numTiePoints = AvhrrConstants.TP_GRID_WIDTH * gridHeight;

		float[][] tiePointData = new float[6][numTiePoints];
		final int numRawAngles = AvhrrConstants.TP_GRID_WIDTH * 4;
		final int numRawLatLon = AvhrrConstants.TP_GRID_WIDTH * 2;

		short[] rawAngles = new short[numRawAngles];
		int[] rawLatLon = new int[numRawLatLon];

		int targetIndex = 0;
		int targetIncr = 1;

		for (int scanLine = 0; scanLine < getProductHeight(); scanLine += AvhrrConstants.TP_SUB_SAMPLING_Y) {
			final int scanLineOffset = getScanLineOffset(scanLine);
			synchronized (inputStream) {
				inputStream.seek(scanLineOffset + 20556);
				inputStream.readFully(rawAngles, 0, numRawAngles);
				inputStream.readFully(rawLatLon, 0, numRawLatLon);
			}
			for (int scanPoint = 0; scanPoint < AvhrrConstants.TP_GRID_WIDTH; scanPoint++) {
				tiePointData[0][targetIndex] = rawAngles[scanPoint * 4] * 1E-2f;
				tiePointData[1][targetIndex] = rawAngles[scanPoint * 4 + 1] * 1E-2f;
				tiePointData[2][targetIndex] = rawAngles[scanPoint * 4 + 2] * 1E-2f;
				tiePointData[3][targetIndex] = rawAngles[scanPoint * 4 + 3] * 1E-2f;

				tiePointData[4][targetIndex] = rawLatLon[scanPoint * 2] * 1E-4f;
				tiePointData[5][targetIndex] = rawLatLon[scanPoint * 2 + 1] * 1E-4f;
				targetIndex += targetIncr;
			}
		}
		return tiePointData;
	}

	public int getScanLineOffset(int rawY) {
		return firstMdrOffset + (rawY * MDR_SIZE); // 26660 MDR-1B size
	}

	public int getFlagOffset(int rawY) {
		return getScanLineOffset(rawY) + 22204 - 832;
	}

	public int readFrameIndicator(int rawY) throws IOException {
		final int flagOffset = getScanLineOffset(rawY) + 26580 - 832 + 1;
		byte flag;
		synchronized (inputStream) {
			inputStream.seek(flagOffset);
			flag = inputStream.readByte();
		}
		return flag;
	}

	public static boolean canOpenFile(File file) throws IOException {
		ImageInputStream inputStream = new FileImageInputStream(file);
		try {
			GenericRecordHeader mphrHeader = new GenericRecordHeader();
			mphrHeader.readGenericRecordHeader(inputStream);

			// check for MPHR 
			if (mphrHeader.recordClass == GenericRecordHeader.RecordClass.MPHR
					&& mphrHeader.instrumentGroup == GenericRecordHeader.InstrumentGroup.GENERIC
					&& mphrHeader.recordSubclass == 0) {
				
				inputStream.seek(mphrHeader.recordSize);
				GenericRecordHeader sphrHeader = new GenericRecordHeader();
				sphrHeader.readGenericRecordHeader(inputStream);

				// check for SPHR and AVHRR/3
				if (sphrHeader.recordClass == GenericRecordHeader.RecordClass.SPHR
						&& sphrHeader.instrumentGroup == GenericRecordHeader.InstrumentGroup.AVHRR3
						&& sphrHeader.recordSubclass == 0) {
					return true;
				}
			}
		} finally {
			inputStream.close();
		}
		return false;
	}

	private void analyzeFrameIndicator() throws IOException {
		final int first = readFrameIndicator(0);
		final int last = readFrameIndicator(getProductHeight() - 1);

		final int firstChannel3ab = first & 1;
		final int lastChannel3ab = last & 1;
		if (firstChannel3ab == 1 && lastChannel3ab == 1) {
			channel3ab = AvhrrConstants.CH_3A;
		} else if (firstChannel3ab == 0 && lastChannel3ab == 0) {
			channel3ab = AvhrrConstants.CH_3B;
		} else {
			channel3ab = -1;
		}
	}
	
	private void checkMdrs() throws IOException {
		GenericRecordHeader firstMdr = new GenericRecordHeader();
		synchronized (inputStream) {
			inputStream.seek(firstMdrOffset);
			firstMdr.readGenericRecordHeader(inputStream);
		}
		if(firstMdr.recordSize != MDR_SIZE) {
			throw new IOException("MDR has wrong size. Expected: " + MDR_SIZE
					+ " Actual MDR record size: " + firstMdr.recordSize);
		}
		
		final long fileSize = inputStream.length();
		final long expectedFileSize = firstMdrOffset + (productHeight * MDR_SIZE);
		if(fileSize != expectedFileSize) {
			throw new IOException("Product has wrong file size. Expected filesize: " + expectedFileSize
					+ " Actual filesize: " + fileSize);
		}
	}
}
