/*
 * AVISA software - $Id: PlainBandReader.java,v 1.3 2006/03/13 14:02:04 marcoz Exp $
 *
 * Copyright (C) 2005 by EUMETSAT
 *
 * The Licensee acknowledges that the AVISA software is owned by the European
 * Organisation for the Exploitation of Meteorological Satellites
 * (EUMETSAT) and the Licensee shall not transfer, assign, sub-licence,
 * reproduce or copy the AVISA software to any third party or part with
 * possession of this software or any part thereof in any way whatsoever.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 * The AVISA software has been developed using the ESA BEAM software which is
 * distributed under the GNU General Public License (GPL).
 *
 */
package org.eumetsat.beam.dataio.metop;

import java.io.IOException;

import javax.imageio.stream.ImageInputStream;

import org.esa.beam.dataio.avhrr.AvhrrConstants;
import org.esa.beam.dataio.avhrr.AvhrrFile;
import org.esa.beam.dataio.avhrr.BandReader;
import org.esa.beam.dataio.avhrr.AvhrrReader;
import org.esa.beam.framework.datamodel.ProductData;

import com.bc.progress.ProgressController;
import com.bc.progress.ProgressControllerPool;

/**
 * Reads radiances directly from the METOP prodcuts.
 * @author marcoz
 *
 */
class PlainBandReader implements BandReader {
	
	private static final String VIS_RADIANCE_UNIT = "W / (m^2 sr)";
	private static final String IR_RADIANCE_UNIT = "mW / (m^2 sr cm)";
	
	protected int channel;

	protected MetopFile metopFile;

	protected ImageInputStream inputStream;

	public PlainBandReader(int channel, MetopFile metopFile,
			ImageInputStream inputStream) {
		this.channel = channel;
		this.metopFile = metopFile;
		this.inputStream = inputStream;
	}

	public String getBandName() {
		return AvhrrConstants.RADIANCE_BAND_NAME_PREFIX
				+ AvhrrConstants.CH_STRINGS[channel];
	}

	public String getBandUnit() {
		if (isVisibleBand()) {
			return VIS_RADIANCE_UNIT;
		} else {
			return IR_RADIANCE_UNIT;
		}
	}

	public String getBandDescription() {
		if (isVisibleBand()) {
			return AvhrrReader.format(AvhrrConstants.RADIANCE_DESCRIPTION_VIS,
					AvhrrConstants.CH_STRINGS[channel]);
		} else {
			return AvhrrReader.format(AvhrrConstants.RADIANCE_DESCRIPTION_IR,
					AvhrrConstants.CH_STRINGS[channel]);
		}
	}

	public float getScalingFactor() {
		if (channel == AvhrrConstants.CH_3A || channel == AvhrrConstants.CH_3B) {
			return 1E-4f;
		} else {
			return 1E-2f;
		}
	}

	public int getDataType() {
		return ProductData.TYPE_INT16;
	}

	public void readBandRasterData(int sourceOffsetX, int sourceOffsetY,
			int sourceWidth, int sourceHeight, int sourceStepX,
			int sourceStepY, ProductData destBuffer) throws IOException {

		AvhrrFile.RawCoordinates rawCoord = metopFile.getRawCoordiantes(
				sourceOffsetX, sourceOffsetY, sourceWidth, sourceHeight);
		final short[] targetData = (short[]) destBuffer.getElems();

		final ProgressController pc = ProgressControllerPool.getInstance()
				.getProgressController();
		pc.fireProcessStarted("Reading AVHRR band '" + getBandName() + "'...",
				rawCoord.minY, rawCoord.maxY); /* I18N */

		int targetIdx = rawCoord.targetStart;
		for (int sourceY = rawCoord.minY; sourceY <= rawCoord.maxY; sourceY += sourceStepY) {
			if (pc.isTerminationRequested()) {
				break;
			}

			if (hasData(sourceY)) {
				final int dataOffset = getDataOffset(sourceOffsetX, sourceY);
				synchronized (inputStream) {
					inputStream.seek(dataOffset);
					inputStream.readFully(targetData, targetIdx, sourceWidth);
				}
				targetIdx += sourceWidth;
			} else {
				for (int sourceX = rawCoord.minX; sourceX <= rawCoord.maxX; sourceX += sourceStepX) {
					targetData[targetIdx] = AvhrrConstants.NO_DATA_VALUE;
					targetIdx += rawCoord.targetIncrement;
				}
			}
			pc.fireProcessInProgress(sourceY);
		}
		pc.fireProcessEnded();

	}

	protected int getDataOffset(int sourceOffsetX, int sourceY) {
		return metopFile.getScanLineOffset(sourceY)
				+ 24
				+ (AvhrrConstants.RAW_SCENE_RASTER_WIDTH * AvhrrConstants.CH_DATASET_INDEXES[channel] * 2)
				+ ((AvhrrConstants.TP_TRIM_X + sourceOffsetX) * 2);
	}

	protected boolean isVisibleBand() {
		if (channel == AvhrrConstants.CH_1 || channel == AvhrrConstants.CH_2
				|| channel == AvhrrConstants.CH_3A) {
			return true;
		} else {
			return false;
		}
	}

	protected boolean hasData(int rawY) throws IOException {
		if (channel != AvhrrConstants.CH_3A && channel != AvhrrConstants.CH_3B) {
			return true;
		}
		final int bitField = metopFile.readFrameIndicator(rawY);
		final int channel3ab = bitField & 1;
		if ((channel3ab == 1 && channel == AvhrrConstants.CH_3A)
				|| (channel3ab == 0 && channel == AvhrrConstants.CH_3B)) {
			return true;
		}
		return false;
	}

	
}